#!/usr/bin/env python
"""Fuel dataset conversion utility."""
import argparse
import importlib
import os
import sys

import h5py

import fuel
from fuel import converters
from fuel.converters.base import MissingInputFiles
from fuel.datasets import H5PYDataset


class CheckDirectoryAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        if os.path.isdir(values):
            setattr(namespace, self.dest, values)
        else:
            raise ValueError('{} is not a existing directory'.format(values))


def main(args=None):
    """Entry point for `fuel-convert` script.

    This function can also be imported and used from Python.

    Parameters
    ----------
    args : iterable, optional (default: None)
        A list of arguments that will be passed to Fuel's conversion
        utility. If this argument is not specified, `sys.argv[1:]` will
        be used.

    """
    built_in_datasets = dict(converters.all_converters)
    if fuel.config.extra_converters:
        for name in fuel.config.extra_converters:
            extra_datasets = dict(
                importlib.import_module(name).all_converters)
            if any(key in built_in_datasets for key in extra_datasets.keys()):
                raise ValueError('extra converters conflict in name with '
                                 'built-in converters')
            built_in_datasets.update(extra_datasets)
    parser = argparse.ArgumentParser(
        description='Conversion script for built-in datasets.')
    subparsers = parser.add_subparsers()
    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "-d", "--directory", help="directory in which input files reside",
        type=str, default=os.getcwd())
    convert_functions = {}
    for name, fill_subparser in built_in_datasets.items():
        subparser = subparsers.add_parser(
            name, parents=[parent_parser],
            help='Convert the {} dataset'.format(name))
        subparser.add_argument(
            "-o", "--output-directory", help="where to save the dataset",
            type=str, default=os.getcwd(), action=CheckDirectoryAction)
        # Allows the parser to know which subparser was called.
        subparser.set_defaults(which_=name)
        convert_functions[name] = fill_subparser(subparser)

    args = parser.parse_args(args)
    args_dict = vars(args)
    convert_function = convert_functions[args_dict.pop('which_')]
    try:
        output_paths = convert_function(**args_dict)
    except MissingInputFiles as e:
        intro = "The following required files were not found:\n"
        message = "\n".join([intro] + ["   * " + f for f in e.filenames])
        message += "\n\nDid you forget to run fuel-download?"
        parser.error(message)

    # Tag the newly-created file(s) with H5PYDataset version and command-line
    # options
    for output_path in output_paths:
        h5file = h5py.File(output_path, 'a')
        interface_version = H5PYDataset.interface_version.encode('utf-8')
        h5file.attrs['h5py_interface_version'] = interface_version
        fuel_convert_version = converters.__version__.encode('utf-8')
        h5file.attrs['fuel_convert_version'] = fuel_convert_version
        command = [os.path.basename(sys.argv[0])] + sys.argv[1:]
        h5file.attrs['fuel_convert_command'] = (
            ' '.join(command).encode('utf-8'))
        h5file.flush()
        h5file.close()


if __name__ == "__main__":
    main()



import os

try:
    import pafy
    PAFY_AVAILABLE = True
except ImportError:
    PAFY_AVAILABLE = False


def download(directory, youtube_id, clear=False):
    """Download the audio of a YouTube video.

    The audio is downloaded in the highest available quality. Progress is
    printed to `stdout`. The file is named `youtube_id.m4a`, where
    `youtube_id` is the 11-character code identifiying the YouTube video
    (can be determined from the URL).

    Parameters
    ----------
    directory : str
        The directory in which to save the downloaded audio file.
    youtube_id : str
        11-character video ID (taken from YouTube URL)
    clear : bool
        If `True`, it deletes the downloaded video. Otherwise it downloads
        it. Defaults to `False`.

    """
    filepath = os.path.join(directory, '{}.m4a'.format(youtube_id))
    if clear:
        os.remove(filepath)
        return
    if not PAFY_AVAILABLE:
        raise ImportError("pafy is required to download YouTube videos")
    url = 'https://www.youtube.com/watch?v={}'.format(youtube_id)
    video = pafy.new(url)
    audio = video.getbestaudio()
    audio.download(quiet=False, filepath=filepath)


def fill_subparser(subparser):
    """Sets up a subparser to download audio of YouTube videos.

    Adds the compulsory `--youtube-id` flag.

    Parameters
    ----------
    subparser : :class:`argparse.ArgumentParser`
        Subparser handling the `youtube_audio` command.

    """
    subparser.add_argument(
        '--youtube-id', type=str, required=True,
        help=("The YouTube ID of the video from which to extract audio, "
              "usually an 11-character string.")
    )
    return download



#!/usr/bin/env python
"""Fuel utility for extracting metadata."""
import argparse
import os

import h5py

message_prefix_template = 'Metadata for {}'
message_body_template = """

    The command used to generate this file is

        {}

    Relevant versions are

        H5PYDataset     {}
        fuel.converters {}
"""


def main(args=None):
    """Entry point for `fuel-info` script.

    This function can also be imported and used from Python.

    Parameters
    ----------
    args : iterable, optional (default: None)
        A list of arguments that will be passed to Fuel's information
        utility. If this argument is not specified, `sys.argv[1:]` will
        be used.

    """
    parser = argparse.ArgumentParser(
        description='Extracts metadata from a Fuel-converted HDF5 file.')
    parser.add_argument("filename", help="HDF5 file to analyze")
    args = parser.parse_args()

    with h5py.File(args.filename, 'r') as h5file:
        interface_version = h5file.attrs.get('h5py_interface_version', 'N/A')
        fuel_convert_version = h5file.attrs.get('fuel_convert_version', 'N/A')
        fuel_convert_command = h5file.attrs.get('fuel_convert_command', 'N/A')

    message_prefix = message_prefix_template.format(
        os.path.basename(args.filename))
    message_body = message_body_template.format(
        fuel_convert_command, interface_version, fuel_convert_version)
    message = ''.join(['\n', message_prefix, '\n', '=' * len(message_prefix),
                       message_body])
    print(message)


if __name__ == "__main__":
    main()



import os
import h5py

from scipy.io import loadmat

from fuel.converters.base import fill_hdf5_file, MissingInputFiles


def convert_silhouettes(size, directory, output_directory,
                        output_file=None):
    """ Convert the CalTech 101 Silhouettes Datasets.

    Parameters
    ----------
    size : {16, 28}
        Convert either the 16x16 or 28x28 sized version of the dataset.
    directory : str
        Directory in which the required input files reside.
    output_file : str
        Where to save the converted dataset.

    """
    if size not in (16, 28):
        raise ValueError('size must be 16 or 28')

    if output_file is None:
        output_file = 'caltech101_silhouettes{}.hdf5'.format(size)
    output_file = os.path.join(output_directory, output_file)

    input_file = 'caltech101_silhouettes_{}_split1.mat'.format(size)
    input_file = os.path.join(directory, input_file)

    if not os.path.isfile(input_file):
        raise MissingInputFiles('Required files missing', [input_file])

    with h5py.File(output_file, mode="w") as h5file:
        mat = loadmat(input_file)

        train_features = mat['train_data'].reshape([-1, 1, size, size])
        train_targets = mat['train_labels']
        valid_features = mat['val_data'].reshape([-1, 1, size, size])
        valid_targets = mat['val_labels']
        test_features = mat['test_data'].reshape([-1, 1, size, size])
        test_targets = mat['test_labels']

        data = (
            ('train', 'features', train_features),
            ('train', 'targets', train_targets),
            ('valid', 'features', valid_features),
            ('valid', 'targets', valid_targets),
            ('test', 'features', test_features),
            ('test', 'targets', test_targets),
        )
        fill_hdf5_file(h5file, data)

        for i, label in enumerate(('batch', 'channel', 'height', 'width')):
            h5file['features'].dims[i].label = label

        for i, label in enumerate(('batch', 'index')):
            h5file['targets'].dims[i].label = label
    return (output_file,)


def fill_subparser(subparser):
    """Sets up a subparser to convert CalTech101 Silhouettes Database files.

    Parameters
    ----------
    subparser : :class:`argparse.ArgumentParser`
        Subparser handling the `caltech101_silhouettes` command.

    """
    subparser.add_argument(
        "size", type=int, choices=(16, 28),
        help="height/width of the datapoints")
    return convert_silhouettes



from fuel.downloaders.base import default_downloader


def fill_subparser(subparser):
    """Sets up a subparser to download the Dogs vs. Cats dataset file.

    Kaggle's Dogs vs. Cats [KAGGLE] dataset is downloaded from Dropbox
    since Kaggle requires user authentication.

    .. [KAGGLE] https://www.kaggle.com/c/dogs-vs-cats

    Parameters
    ----------
    subparser : :class:`argparse.ArgumentParser`
        Subparser handling the `dogs_vs_cats` command.

    """
    urls = ['https://www.dropbox.com/s/s3u30quvpxqdbz6/train.zip?dl=1',
            'https://www.dropbox.com/s/21rwu6drnplsbkb/test1.zip?dl=1']
    filenames = ['dogs_vs_cats.train.zip', 'dogs_vs_cats.test1.zip']
    subparser.set_defaults(urls=urls, filenames=filenames)
    return default_downloader



import os
import subprocess
import sys

import h5py
import scipy.io.wavfile

from fuel.converters.base import fill_hdf5_file


def convert_youtube_audio(directory, output_directory, youtube_id, channels,
                          sample, output_filename=None):
    """Converts downloaded YouTube audio to HDF5 format.

    Requires `ffmpeg` to be installed and available on the command line
    (i.e. available on your `PATH`).

    Parameters
    ----------
    directory : str
        Directory in which input files reside.
    output_directory : str
        Directory in which to save the converted dataset.
    youtube_id : str
        11-character video ID (taken from YouTube URL)
    channels : int
        The number of audio channels to use in the PCM Wave file.
    sample : int
        The sampling rate to use in Hz, e.g. 44100 or 16000.
    output_filename : str, optional
        Name of the saved dataset. If `None` (the default),
        `youtube_id.hdf5` is used.

    """
    input_file = os.path.join(directory, '{}.m4a'.format(youtube_id))
    wav_filename = '{}.wav'.format(youtube_id)
    wav_file = os.path.join(directory, wav_filename)
    ffmpeg_not_available = subprocess.call(['ffmpeg', '-version'])
    if ffmpeg_not_available:
        raise RuntimeError('conversion requires ffmpeg')
    subprocess.check_call(['ffmpeg', '-y', '-i', input_file, '-ac',
                           str(channels), '-ar', str(sample), wav_file],
                          stdout=sys.stdout)

    # Load WAV into array
    _, data = scipy.io.wavfile.read(wav_file)
    if data.ndim == 1:
        data = data[:, None]
    data = data[None, :]

    # Store in HDF5
    if output_filename is None:
        output_filename = '{}.hdf5'.format(youtube_id)
    output_file = os.path.join(output_directory, output_filename)

    with h5py.File(output_file, 'w') as h5file:
        fill_hdf5_file(h5file, (('train', 'features', data),))
        h5file['features'].dims[0].label = 'batch'
        h5file['features'].dims[1].label = 'time'
        h5file['features'].dims[2].label = 'feature'

    return (output_file,)


def fill_subparser(subparser):
    """Sets up a subparser to convert YouTube audio files.

    Adds the compulsory `--youtube-id` flag as well as the optional
    `sample` and `channels` flags.

    Parameters
    ----------
    subparser : :class:`argparse.ArgumentParser`
        Subparser handling the `youtube_audio` command.

    """
    subparser.add_argument(
        '--youtube-id', type=str, required=True,
        help=("The YouTube ID of the video from which to extract audio, "
              "usually an 11-character string.")
    )
    subparser.add_argument(
        '--channels', type=int, default=1,
        help=("The number of audio channels to convert to. The default of 1"
              "means audio is converted to mono.")
    )
    subparser.add_argument(
        '--sample', type=int, default=16000,
        help=("The sampling rate in Hz. The default of 16000 is "
              "significantly downsampled compared to normal WAVE files; "
              "pass 44100 for the usual sampling rate.")
    )
    return convert_youtube_audio



"""Download modules for built-in datasets.

Download functions accept two arguments:

* `save_directory` : Where to save the downloaded files
* `clear` : If `True`, clear the downloaded files. Defaults to `False`.

"""
from fuel.downloaders import adult
from fuel.downloaders import binarized_mnist
from fuel.downloaders import caltech101_silhouettes
from fuel.downloaders import celeba
from fuel.downloaders import cifar10
from fuel.downloaders import cifar100
from fuel.downloaders import dogs_vs_cats
from fuel.downloaders import iris
from fuel.downloaders import mnist
from fuel.downloaders import svhn
from fuel.downloaders import ilsvrc2010
from fuel.downloaders import youtube_audio

all_downloaders = (
    ('adult', adult.fill_subparser),
    ('binarized_mnist', binarized_mnist.fill_subparser),
    ('caltech101_silhouettes', caltech101_silhouettes.fill_subparser),
    ('celeba', celeba.fill_subparser),
    ('cifar10', cifar10.fill_subparser),
    ('cifar100', cifar100.fill_subparser),
    ('iris', iris.fill_subparser),
    ('mnist', mnist.fill_subparser),
    ('svhn', svhn.fill_subparser),
    ('ilsvrc2010', ilsvrc2010.fill_subparser),
    ('dogs_vs_cats', dogs_vs_cats.fill_subparser),
    ('youtube_audio', youtube_audio.fill_subparser))



"""Utilities for speeding things up through parallelism.

Currently including:

* A very simple PUSH-PULL reusable producer-consumer pattern
  using a ZeroMQ socket instead of the (slow, unnecessarily
  copying) multiprocessing.Queue. See :func:`producer_consumer`.

"""
from multiprocessing import Process
import zmq


def _producer_wrapper(f, port, addr='tcp://127.0.0.1'):
    """A shim that sets up a socket and starts the producer callable.

    Parameters
    ----------
    f : callable
        Callable that takes a single argument, a handle
        for a ZeroMQ PUSH socket. Must be picklable.
    port : int
        The port on which the socket should connect.
    addr : str, optional
        Address to which the socket should connect. Defaults
        to localhost ('tcp://127.0.0.1').

    """
    try:
        context = zmq.Context()
        socket = context.socket(zmq.PUSH)
        socket.connect(':'.join([addr, str(port)]))
        f(socket)
    finally:
        # Works around a Python 3.x bug.
        context.destroy()


def _spawn_producer(f, port, addr='tcp://127.0.0.1'):
    """Start a process that sends results on a PUSH socket.

    Parameters
    ----------
    f : callable
        Callable that takes a single argument, a handle
        for a ZeroMQ PUSH socket. Must be picklable.

    Returns
    -------
    process : multiprocessing.Process
        The process handle of the created producer process.

    """
    process = Process(target=_producer_wrapper, args=(f, port, addr))
    process.start()
    return process


def producer_consumer(producer, consumer, addr='tcp://127.0.0.1',
                      port=None, context=None):
    """A producer-consumer pattern.

    Parameters
    ----------
    producer : callable
        Callable that takes a single argument, a handle
        for a ZeroMQ PUSH socket. Must be picklable.
    consumer : callable
        Callable that takes a single argument, a handle
        for a ZeroMQ PULL socket.
    addr : str, optional
        Address to which the socket should connect. Defaults
        to localhost ('tcp://127.0.0.1').
    port : int, optional
        The port on which the consumer should listen.
    context : zmq.Context, optional
        The ZeroMQ Context to use. One will be created otherwise.

    Returns
    -------
    result
        Passes along whatever `consumer` returns.

    Notes
    -----
    This sets up a PULL socket in the calling process and forks
    a process that calls `producer` on a PUSH socket. When the
    consumer returns, the producer process is terminated.

    Wrap `consumer` or `producer` in a `functools.partial` object
    in order to send additional arguments; the callables passed in
    should expect only one required, positional argument, the socket
    handle.

    """
    context_created = False
    if context is None:
        context_created = True
        context = zmq.Context()
    try:
        consumer_socket = context.socket(zmq.PULL)
        if port is None:
            port = consumer_socket.bind_to_random_port(addr)
        try:
            process = _spawn_producer(producer, port)
            result = consumer(consumer_socket)
        finally:
            process.terminate()
        return result
    finally:
        # Works around a Python 3.x bug.
        if context_created:
            context.destroy()



import os

import h5py
import numpy

from fuel.converters.base import fill_hdf5_file, check_exists


TRAIN_FILE = 'binarized_mnist_train.amat'
VALID_FILE = 'binarized_mnist_valid.amat'
TEST_FILE = 'binarized_mnist_test.amat'

ALL_FILES = [TRAIN_FILE, VALID_FILE, TEST_FILE]


@check_exists(required_files=ALL_FILES)
def convert_binarized_mnist(directory, output_directory,
                            output_filename='binarized_mnist.hdf5'):
    """Converts the binarized MNIST dataset to HDF5.

    Converts the binarized MNIST dataset used in R. Salakhutdinov's DBN
    paper [DBN] to an HDF5 dataset compatible with
    :class:`fuel.datasets.BinarizedMNIST`. The converted dataset is
    saved as 'binarized_mnist.hdf5'.

    This method assumes the existence of the files
    `binarized_mnist_{train,valid,test}.amat`, which are accessible
    through Hugo Larochelle's website [HUGO].

    .. [DBN] Ruslan Salakhutdinov and Iain Murray, *On the Quantitative
       Analysis of Deep Belief Networks*, Proceedings of the 25th
       international conference on Machine learning, 2008, pp. 872-879.

    Parameters
    ----------
    directory : str
        Directory in which input files reside.
    output_directory : str
        Directory in which to save the converted dataset.
    output_filename : str, optional
        Name of the saved dataset. Defaults to 'binarized_mnist.hdf5'.

    Returns
    -------
    output_paths : tuple of str
        Single-element tuple containing the path to the converted dataset.

    """
    output_path = os.path.join(output_directory, output_filename)
    h5file = h5py.File(output_path, mode='w')

    train_set = numpy.loadtxt(
        os.path.join(directory, TRAIN_FILE)).reshape(
            (-1, 1, 28, 28)).astype('uint8')
    valid_set = numpy.loadtxt(
        os.path.join(directory, VALID_FILE)).reshape(
            (-1, 1, 28, 28)).astype('uint8')
    test_set = numpy.loadtxt(
        os.path.join(directory, TEST_FILE)).reshape(
            (-1, 1, 28, 28)).astype('uint8')
    data = (('train', 'features', train_set),
            ('valid', 'features', valid_set),
            ('test', 'features', test_set))
    fill_hdf5_file(h5file, data)
    for i, label in enumerate(('batch', 'channel', 'height', 'width')):
        h5file['features'].dims[i].label = label

    h5file.flush()
    h5file.close()

    return (output_path,)


def fill_subparser(subparser):
    """Sets up a subparser to convert the binarized MNIST dataset files.

    Parameters
    ----------
    subparser : :class:`argparse.ArgumentParser`
        Subparser handling the `binarized_mnist` command.

    """
    return convert_binarized_mnist
